unit SOCKS;

{ TBinarySocksClient decendent of TBinaryClient
  Derived object of TBinaryClient that supports connections
  through SOCKS v4 servers.
  (C) Phil Fong 1997 (pfong@geocities.com)
  v 0.06 BETA
  For more information see: HTTP://www.geocities.com/SiliconValley/Heights/2517
  This component is for NON-Commercial use only.
  The Author cannot be held responsible for anything resulting from
  this software.
}

interface

uses dWinsock, buffsock, classes, sysutils, wsapi;

type
    iptype = packed array[1..4] of byte;
    SOCKSDataType = packed record
                ver      : byte;  {set to 4 for SOCKS v4}
                cmd      : byte;
                port     : word;
                destip   : iptype;
                n : byte; {set to 0 when initializing}
    end;
    TBinarySocksClient  = class(TBinaryClient)
                        private
                        FUseSocks : boolean;
                        FAddress  : ShortString;
                        FPort : ShortString;
                        FSocksAddress : ShortString;
                        FSocksPort : ShortString;
                        FOnSocksConnect : TSocketEvent;
                        {FOnSocksRead : TSocketEvent;}
                        Waiting : boolean;
                        procedure SocksConnect(Sender : TObject; Socket : TSocketBase);
                        {procedure SocksRead(Sender : TObject; Socket : TSocketBase);}
                        procedure SetRealPort(p : ShortString);
                        procedure SetRealAddress(A : shortString);
                        procedure SetUseSocks(u : boolean);
                        published
                        constructor Create(AOwner : TComponent); override;
                        procedure Open(ASocketClass : TSocketClass);
                        property UseSocks : boolean read FUseSocks write SetUseSocks default false;
                        property Address : ShortString read FAddress write SetRealAddress;
                        property Port : ShortString read FPort write SetRealPort;
                        property OnConnect : TSocketEvent read FOnSocksConnect write FOnSocksConnect;
                        {property OnRead : TSocketEvent read FOnSocksRead write FOnSocksRead;}
                        property SocksAddress : ShortString read FSocksAddress write FSocksAddress;
                        property SocksPort : ShortString read FSocksPort write FSocksPort;
    end;
procedure register;

implementation

procedure Register;
begin
  RegisterComponents('dWinsock', [TBinarySocksClient]);
end;

constructor TBinarySocksClient.Create(AOwner : TComponent);
begin
     Inherited Create(AOwner);
     inherited OnConnect:=SocksConnect;
     {inherited OnRead:=SocksRead;}
     UseSocks:=false;
end;

procedure TBinarySocksClient.Open(ASocketClass : TSocketClass);
Begin
     if FUseSocks=true then
     begin
          Inherited Port:=FSocksPort;
          Inherited Address:=FSocksAddress;
     end
     else
     begin
          inherited Port:=FPort;
          inherited Address:=FAddress;
     end;
     inherited Open(ASocketClass);
end;

procedure TBinarySocksClient.SocksConnect(Sender : TObject; Socket : TSocketBase);
var
   SocksData : SocksDataType;
   temp : integer;
   Stemp : shortstring;
   TAddress:shortstring;
   TOutTime:TDateTime;

begin
     If conn.state=ssConnected then
     begin
          If UseSocks=true then
          begin
               SocksData.ver:=4;
               SocksData.cmd:=1;
               SocksData.n:=0;
               If WinSockAPI.IsPortNumber(Fport) then
                  SocksData.Port:=StrToInt(FPort)
               else
                   SocksData.Port:=LookupPortByService(FPort);
               SocksData.Port:=WinsockAPI.htons(SocksData.Port);
               if WinSockAPI.IsInternetAddress(FAddress)<>True then
                  TAddress:=LookupAddrByNameStr(FAddress)
               else
                   TAddress:=FAddress;
               temp:=pos('.',TAddress)-1;
               SocksData.Destip[1]:=StrToInt(Copy(TAddress,1,temp));
               stemp:=Copy(TAddress,temp+2,length(TAddress)-temp+2);
               temp:=pos('.',stemp)-1;
               SocksData.Destip[2]:=StrToInt(Copy(Stemp,1,temp));
               stemp:=Copy(stemp,temp+2,length(stemp)-temp+2);
               temp:=pos('.',stemp)-1;
               SocksData.Destip[3]:=StrToInt(Copy(Stemp,1,temp));
               stemp:=copy(Stemp,temp+2,length(stemp)-temp+2);
               SocksData.Destip[4]:=StrToInt(stemp);
               (conn as TBufferedSocket).WriteToBuffer(socksData,sizeof(socksdata));
               (conn as TBufferedSocket).SendBuffer;
               If TimeOut>0 then TOutTime:=time+(TimeOut/86400) else TOutTime:=1.7e308;
               While((conn.connected) and ((conn as TBufferedSocket).ReadBufferBytesUsed<sizeof(SocksData)-1)) do
               begin
                    (conn as TBufferedSocket).RecvBuffer;
                    If TOutTime<Time then
                    begin
                    {TimeOut!!}
                         socket.Close;
                         If assigned(OnTimeOut) then OnTimeOut(Sender,Socket);
                         exit;
                    end;
               end;
               (conn as TBufferedSocket).ReadfromBuffer(socksdata,sizeof(socksdata)-1);
               If (Socksdata.ver<>0) and (SocksData.cmd<>90) then
               begin
                    socket.close;
                    waiting:=false;
                    exit;
               end;
          end;
     end;
     waiting:=false;
     If Assigned(FOnSocksConnect) then FOnSocksConnect(Sender, Socket);
     If (conn as TBufferedSocket).ReadBufferBytesUsed<>0 then
        if assigned(OnRead) then OnRead(Sender,Socket);
end;

procedure TBinarySocksClient.SetRealAddress(a : ShortString);
begin
     FAddress:=a;
     If FUseSocks=false then
     begin
          inherited address:=a;
     end
     else
     begin
          inherited address:=FSocksAddress;
     end;
end;

procedure TBinarySocksClient.SetRealPort(p : shortString);
begin
     Fport:=p;
     If FUseSocks=false then
     begin
          inherited port:=p;
     end
     else
     begin
          inherited port:=FSocksPort;
     end;
end;

procedure TBinarySocksClient.SetUseSocks(u : boolean);
begin
     If connected=true then exit;
     FUseSocks:=u;
end;

{procedure TBinarySocksClient.SocksRead(Sender : TObject; Socket : TSocketBase);
begin
     If waiting=true then exit;
     If Assigned(FOnSocksRead) then FOnSocksRead(Sender,Socket);
end;}


end.
